"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  H: () => H,
  HIGHLIGHT_REQUEST_HEADER: () => HIGHLIGHT_REQUEST_HEADER,
  Handlers: () => handlers_exports,
  Highlight: () => Highlight
});
module.exports = __toCommonJS(src_exports);

// src/log.ts
var start = Date.now();
function log(from, ...data) {
  const prefix = `[Highlight Node.js ${(Date.now() - start) / 1e3}s] - {${from}}`;
  console.log.apply(console, [prefix, ...data]);
}

// src/sdk.ts
var HIGHLIGHT_REQUEST_HEADER = "x-highlight-request";
var _debug = false;
var highlight_obj;
var H = {
  init: (options) => {
    _debug = !!options.debug;
    try {
      highlight_obj = new Highlight(options);
    } catch (e) {
      console.warn("highlight-node init error: ", e);
    }
  },
  isInitialized: () => !!highlight_obj,
  consumeError: (error, secureSessionId, requestId) => {
    try {
      highlight_obj.consumeCustomError(error, secureSessionId, requestId);
    } catch (e) {
      console.warn("highlight-node consumeError error: ", e);
    }
  },
  consumeEvent: (secureSessionId) => {
    try {
      highlight_obj.consumeCustomEvent(secureSessionId);
    } catch (e) {
      console.warn("highlight-node consumeEvent error: ", e);
    }
  },
  recordMetric: (secureSessionId, name, value, requestId, tags) => {
    try {
      highlight_obj.recordMetric(
        secureSessionId,
        name,
        value,
        requestId,
        tags
      );
    } catch (e) {
      console.warn("highlight-node recordMetric error: ", e);
    }
  },
  flush: async () => {
    try {
      await highlight_obj.flush();
    } catch (e) {
      console.warn("highlight-node flush error: ", e);
    }
  },
  log: (message, level, secureSessionId, requestId) => {
    const o = { stack: {} };
    Error.captureStackTrace(o);
    try {
      highlight_obj.log(
        new Date(),
        message,
        level,
        o.stack,
        secureSessionId,
        requestId
      );
    } catch (e) {
      console.warn("highlight-node log error: ", e);
    }
  },
  parseHeaders: (headers) => {
    try {
      if (headers && headers[HIGHLIGHT_REQUEST_HEADER]) {
        const [secureSessionId, requestId] = `${headers[HIGHLIGHT_REQUEST_HEADER]}`.split("/");
        return { secureSessionId, requestId };
      } else {
        H._debug(
          `request headers do not contain ${HIGHLIGHT_REQUEST_HEADER}`
        );
      }
    } catch (e) {
      console.warn("highlight-node parseHeaders error: ", e);
    }
    return void 0;
  },
  _debug: (...data) => {
    if (_debug) {
      log("H", ...data);
    }
  }
};

// src/graph/generated/operations.ts
var import_graphql_tag = __toESM(require("graphql-tag"));
var MarkBackendSetupDocument = import_graphql_tag.default`
	mutation MarkBackendSetup(
		$project_id: String!
		$session_secure_id: String
	) {
		markBackendSetup(
			project_id: $project_id
			session_secure_id: $session_secure_id
		)
	}
`;
var PushMetricsDocument = import_graphql_tag.default`
	mutation PushMetrics($metrics: [MetricInput]!) {
		pushMetrics(metrics: $metrics)
	}
`;
var defaultWrapper = (action, _operationName, _operationType) => action();
function getSdk(client, withWrapper = defaultWrapper) {
  return {
    MarkBackendSetup(variables, requestHeaders) {
      return withWrapper(
        (wrappedRequestHeaders) => client.request(
          MarkBackendSetupDocument,
          variables,
          { ...requestHeaders, ...wrappedRequestHeaders }
        ),
        "MarkBackendSetup",
        "mutation"
      );
    },
    PushMetrics(variables, requestHeaders) {
      return withWrapper(
        (wrappedRequestHeaders) => client.request(
          PushMetricsDocument,
          variables,
          { ...requestHeaders, ...wrappedRequestHeaders }
        ),
        "PushMetrics",
        "mutation"
      );
    }
  };
}

// src/client.ts
var import_graphql_request = require("graphql-request");
var import_auto_instrumentations_node = require("@opentelemetry/auto-instrumentations-node");
var import_exporter_trace_otlp_http = require("@opentelemetry/exporter-trace-otlp-http");
var import_api = require("@opentelemetry/api");

// src/hooks.ts
var ConsoleLevels = {
  debug: "debug",
  info: "info",
  log: "info",
  count: "info",
  dir: "info",
  warn: "warn",
  assert: "warn",
  error: "error",
  trace: "trace"
};
var consoleHooked = false;
function hookConsole(methodsToRecord, cb) {
  if (consoleHooked)
    return;
  consoleHooked = true;
  for (const [level, highlightLevel] of Object.entries(ConsoleLevels)) {
    if ((methodsToRecord == null ? void 0 : methodsToRecord.length) && methodsToRecord.indexOf(level) === -1) {
      continue;
    }
    const origWrite = console[level];
    console[level] = function(...data) {
      const date = new Date();
      try {
        return origWrite(...data);
      } finally {
        const o = { stack: {} };
        Error.captureStackTrace(o);
        cb({
          date,
          level: highlightLevel,
          message: data.join(" "),
          stack: o.stack
        });
      }
    };
  }
}

// src/client.ts
var import_sdk_trace_base = require("@opentelemetry/sdk-trace-base");
var import_sdk_node = require("@opentelemetry/sdk-node");
var OTLP_HTTP = "https://otel.highlight.io:4318";
var Highlight = class {
  constructor(options) {
    this.FLUSH_TIMEOUT = 10;
    this.BACKEND_SETUP_TIMEOUT = 15 * 60 * 1e3;
    this.metrics = [];
    this.lastBackendSetupEvent = 0;
    this._debug = !!options.debug;
    this._projectID = options.projectID;
    this._backendUrl = options.backendUrl || "https://pub.highlight.run";
    if (!options.disableConsoleRecording) {
      hookConsole(options.consoleMethodsToRecord, (c) => {
        this.log(c.date, c.message, c.level, c.stack);
      });
    }
    const client = new import_graphql_request.GraphQLClient(this._backendUrl, {
      headers: {}
    });
    this._graphqlSdk = getSdk(client);
    this.tracer = import_api.trace.getTracer("highlight-node");
    const exporter = new import_exporter_trace_otlp_http.OTLPTraceExporter({
      url: `${options.otlpEndpoint ?? OTLP_HTTP}/v1/traces`
    });
    this.processor = new import_sdk_trace_base.BatchSpanProcessor(exporter, {});
    this.otel = new import_sdk_node.NodeSDK({
      autoDetectResources: true,
      defaultAttributes: { "highlight.project_id": this._projectID },
      spanProcessor: this.processor,
      traceExporter: exporter,
      instrumentations: [(0, import_auto_instrumentations_node.getNodeAutoInstrumentations)()]
    });
    this.otel.start();
    this._intervalFunction = setInterval(
      () => this.flush(),
      this.FLUSH_TIMEOUT * 1e3
    );
    this._graphqlSdk.MarkBackendSetup({
      project_id: this._projectID
    }).then(() => {
      this.lastBackendSetupEvent = Date.now();
    }).catch((e) => {
      console.warn("highlight-node error: ", e);
    });
    this._log(`Initialized SDK for project ${this._projectID}`);
  }
  _log(...data) {
    if (this._debug) {
      log("client", ...data);
    }
  }
  recordMetric(secureSessionId, name, value, requestId, tags) {
    this.metrics.push({
      session_secure_id: secureSessionId,
      group: requestId,
      name,
      value,
      category: "BACKEND",
      timestamp: new Date().toISOString(),
      tags
    });
  }
  log(date, msg, level, stack, secureSessionId, requestId) {
    if (!this.tracer)
      return;
    const span = this.tracer.startSpan("highlight-ctx");
    span.addEvent(
      "log",
      {
        ["exception.stacktrace"]: JSON.stringify(stack),
        ["highlight.project_id"]: this._projectID,
        ["log.severity"]: level,
        ["log.message"]: msg,
        ...secureSessionId ? {
          ["highlight.session_id"]: secureSessionId
        } : {},
        ...requestId ? {
          ["highlight.trace_id"]: requestId
        } : {}
      },
      date
    );
    span.end();
  }
  consumeCustomError(error, secureSessionId, requestId) {
    let span = import_api.trace.getActiveSpan();
    let spanCreated = false;
    if (!span) {
      span = this.tracer.startSpan("highlight-ctx");
      spanCreated = true;
    }
    span.recordException(error);
    span.setAttributes({ ["highlight.project_id"]: this._projectID });
    if (secureSessionId) {
      span.setAttributes({ ["highlight.session_id"]: secureSessionId });
    }
    if (requestId) {
      span.setAttributes({ ["highlight.trace_id"]: requestId });
    }
    if (spanCreated) {
      this._log("created error span", span);
      span.end();
    } else {
      this._log("updated current span with error", span);
    }
  }
  consumeCustomEvent(secureSessionId) {
    const sendBackendSetup = Date.now() - this.lastBackendSetupEvent > this.BACKEND_SETUP_TIMEOUT;
    if (sendBackendSetup) {
      this._graphqlSdk.MarkBackendSetup({
        project_id: this._projectID,
        session_secure_id: secureSessionId
      }).then(() => {
        this.lastBackendSetupEvent = Date.now();
      }).catch((e) => {
        console.warn("highlight-node error: ", e);
      });
    }
  }
  async flushMetrics() {
    if (this.metrics.length === 0) {
      return;
    }
    const variables = {
      metrics: this.metrics
    };
    this.metrics = [];
    try {
      await this._graphqlSdk.PushMetrics(variables);
    } catch (e) {
      console.warn("highlight-node pushMetrics error: ", e);
    }
  }
  async flush() {
    await Promise.all([this.flushMetrics(), this.processor.forceFlush()]);
  }
};

// src/handlers.ts
var handlers_exports = {};
__export(handlers_exports, {
  errorHandler: () => errorHandler,
  firebaseCallableFunctionHandler: () => firebaseCallableFunctionHandler,
  firebaseHttpFunctionHandler: () => firebaseHttpFunctionHandler,
  trpcOnError: () => trpcOnError
});
function processErrorImpl(options, req, error) {
  let secureSessionId;
  let requestId;
  if (req.headers && req.headers[HIGHLIGHT_REQUEST_HEADER]) {
    ;
    [secureSessionId, requestId] = `${req.headers[HIGHLIGHT_REQUEST_HEADER]}`.split("/");
  }
  H._debug("processError", "extracted from headers", {
    secureSessionId,
    requestId
  });
  if (!H.isInitialized()) {
    H.init(options);
    H._debug("initialized H");
  }
  H.consumeEvent(secureSessionId);
  H.consumeError(error, secureSessionId, requestId);
  H._debug("consumed error", error);
}
function errorHandler(options) {
  H._debug("setting up error handler");
  return (error, req, res, next) => {
    H._debug("handling request");
    try {
      processErrorImpl(options, req, error);
    } finally {
      next(error);
    }
  };
}
async function trpcOnError({
  error,
  req
}, options) {
  try {
    if (!H.isInitialized()) {
      H.init(options);
    }
    processErrorImpl(options, req, error);
    await H.flush();
  } catch (e) {
    console.warn("highlight-node trpcOnError error:", e);
  }
}
function firebaseHttpFunctionHandler(origHandler, options) {
  return async (req, res) => {
    try {
      return await origHandler(req, res);
    } catch (e) {
      try {
        if (e instanceof Error) {
          if (!H.isInitialized()) {
            H.init(options);
          }
          processErrorImpl(options, req, e);
          await H.flush();
        }
      } catch (e2) {
        console.warn(
          "highlight-node firebaseHttpFunctionHandler error:",
          e2
        );
      }
      throw e;
    }
  };
}
function firebaseCallableFunctionHandler(origHandler, options) {
  return async (data, context) => {
    try {
      return await origHandler(data, context);
    } catch (e) {
      try {
        if (e instanceof Error) {
          if (!H.isInitialized()) {
            H.init(options);
          }
          processErrorImpl(options, context.rawRequest, e);
          await H.flush();
        }
      } catch (e2) {
        console.warn(
          "highlight-node firebaseCallableFunctionHandler error:",
          e2
        );
      }
      throw e;
    }
  };
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  H,
  HIGHLIGHT_REQUEST_HEADER,
  Handlers,
  Highlight
});
